@@ -1,10 +1,10 @@
 //
-// $Id: indexer.cpp 4796 2014-09-01 19:45:49Z kevg $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -28,11 +28,13 @@
 #if USE_WINDOWS
 	#define snprintf	_snprintf
 	#define popen		_popen
+	#define RMODE "rb"
 
 	#include <io.h>
 	#include <tlhelp32.h>
 #else
 	#include <unistd.h>
+	#define RMODE "r"
 #endif
 
 /////////////////////////////////////////////////////////////////////////////
@@ -782,7 +784,7 @@ CSphSource * SpawnSourceXMLPipe ( const CSphConfigSection & hSource, const char
 		return NULL;
 	}
 
-	FILE * pPipe = popen ( hSource [ "xmlpipe_command" ].cstr(), "r" );
+	FILE * pPipe = popen ( hSource [ "xmlpipe_command" ].cstr(), RMODE );
 	if ( !pPipe )
 	{
 		fprintf ( stdout, "ERROR: xmlpipe: failed to popen '%s'", hSource [ "xmlpipe_command" ].cstr() );
@@ -820,7 +822,7 @@ CSphSource * SpawnSourceTSVPipe ( const CSphConfigSection & hSource, const char
 		return NULL;
 	}
 
-	FILE * pPipe = popen ( hSource [ "tsvpipe_command" ].cstr(), "r" );
+	FILE * pPipe = popen ( hSource [ "tsvpipe_command" ].cstr(), RMODE );
 	if ( !pPipe )
 	{
 		fprintf ( stdout, "ERROR: tsvpipe: failed to popen '%s'", hSource [ "tsvpipe_command" ].cstr() );
@@ -845,7 +847,7 @@ CSphSource * SpawnSourceCSVPipe ( const CSphConfigSection & hSource, const char
 		return NULL;
 	}
 
-	FILE * pPipe = popen ( hSource [ "csvpipe_command" ].cstr(), "r" );
+	FILE * pPipe = popen ( hSource [ "csvpipe_command" ].cstr(), RMODE );
 	if ( !pPipe )
 	{
 		fprintf ( stdout, "ERROR: csvpipe: failed to popen '%s'", hSource [ "csvpipe_command" ].cstr() );
@@ -992,13 +994,12 @@ bool DoIndex ( const CSphConfigSection & hIndex, const char * sIndexName,
 		if ( !tSettings.m_sIndexTokenFilter.IsEmpty() )
 		{
 			pTokenizer = ISphTokenizer::CreatePluginFilter ( pTokenizer, tSettings.m_sIndexTokenFilter, sError );
+			// need token_filter that just passes init phase in case stopwords or wordforms will be loaded
 			if ( !pTokenizer )
 				sphDie ( "index '%s': %s", sIndexName, sError.cstr() );
 		}
 
 		// multiforms filter
-		sphConfDictionary ( hIndex, tDictSettings );
-
 		pDict = tDictSettings.m_bWordDict
 			? sphCreateDictionaryKeywords ( tDictSettings, NULL, pTokenizer, sIndexName, sError )
 			: sphCreateDictionaryCRC ( tDictSettings, NULL, pTokenizer, sIndexName, sError );
@@ -1255,7 +1256,8 @@ bool DoIndex ( const CSphConfigSection & hIndex, const char * sIndexName,
 		pIndex->SetFieldFilter ( pFieldFilter );
 		pIndex->SetTokenizer ( pTokenizer );
 		pIndex->SetDictionary ( pDict );
-		pIndex->SetKeepAttrs ( g_bKeepAttrs );
+		if ( g_bKeepAttrs )
+			pIndex->SetKeepAttrs ( hIndex["path"].strval() );
 		pIndex->Setup ( tSettings );
 
 		bOK = pIndex->Build ( dSources, g_iMemLimit, g_iWriteBuffer )!=0;
@@ -1682,7 +1684,6 @@ int main ( int argc, char ** argv )
 		} else if ( strcasecmp ( argv[i], "--quiet" )==0 )
 		{
 			g_bQuiet = true;
-			sphSetQuiet ( true );
 
 		} else if ( strcasecmp ( argv[i], "--noprogress" )==0 )
 		{
@@ -1972,19 +1973,24 @@ int main ( int argc, char ** argv )
 	// rotating searchd indices
 	////////////////////////////
 
+	int iExitCode = bIndexedOk ? 0 : 1;
+
 	if ( bIndexedOk && g_bRotate )
 	{
 		if ( !SendRotate ( hConf, true ) )
+		{
 			fprintf ( stdout, "WARNING: indices NOT rotated.\n" );
+			iExitCode = 2;
+		}
 	}
 
 #if SPH_DEBUG_LEAKS
 	sphAllocsStats ();
 #endif
 
-	return bIndexedOk ? 0 : 1;
+	return iExitCode;
 }
 
 //
-// $Id: indexer.cpp 4796 2014-09-01 19:45:49Z kevg $
+// $Id$
 //
