@@ -1,10 +1,10 @@
 //
-// $Id: sphinxexpr.h 4787 2014-08-27 10:22:04Z tomat $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -204,6 +204,18 @@ enum
 	SPH_FACTOR_JSON_OUT		= 1 << 2
 };
 
+
+/// known collations
+enum ESphCollation
+{
+	SPH_COLLATION_LIBC_CI,
+	SPH_COLLATION_LIBC_CS,
+	SPH_COLLATION_UTF8_GENERAL_CI,
+	SPH_COLLATION_BINARY,
+
+	SPH_COLLATION_DEFAULT = SPH_COLLATION_LIBC_CI
+};
+
 /// parses given expression, builds evaluator
 /// returns NULL and fills sError on failure
 /// returns pointer to evaluator on success
@@ -212,7 +224,7 @@ enum
 /// fills pEvalStage with a required (!) evaluation stage
 class CSphQueryProfile;
 ISphExpr * sphExprParse ( const char * sExpr, const ISphSchema & tSchema, ESphAttr * pAttrType, bool * pUsesWeight,
-	CSphString & sError, CSphQueryProfile * pProfiler, ISphExprHook * pHook=NULL,
+	CSphString & sError, CSphQueryProfile * pProfiler, ESphCollation eCollation=SPH_COLLATION_DEFAULT, ISphExprHook * pHook=NULL,
 	bool * pZonespanlist=NULL, DWORD * pPackedFactorsFlags=NULL, ESphEvalStage * pEvalStage=NULL );
 
 //////////////////////////////////////////////////////////////////////////
@@ -238,5 +250,5 @@ float GeodistAdaptiveRad ( float lat1, float lon1, float lat2, float lon2 );
 #endif // _sphinxexpr_
 
 //
-// $Id: sphinxexpr.h 4787 2014-08-27 10:22:04Z tomat $
+// $Id$
 //
