@@ -1,10 +1,10 @@
 //
-// $Id: sphinxint.h 4793 2014-08-29 08:42:54Z joric $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -44,6 +44,8 @@
 #define MVA_OFFSET_MASK		0x7fffffffUL	// MVA offset mask
 #define MVA_ARENA_FLAG		0x80000000UL	// MVA global-arena flag
 
+#define DEFAULT_MAX_MATCHES 1000
+
 //////////////////////////////////////////////////////////////////////////
 
 const DWORD		INDEX_MAGIC_HEADER			= 0x58485053;		///< my magic 'SPHX' header
@@ -302,7 +304,6 @@ public:
 	DWORD		UnzipInt ();
 	uint64_t	UnzipOffset ();
 
-	SphOffset_t				Tell () const				{ return m_iPos + m_iBuffPos; }
 	bool					GetErrorFlag () const		{ return m_bError; }
 	const CSphString &		GetErrorMessage () const	{ return m_sError; }
 	const CSphString &		GetFilename() const			{ return m_sFilename; }
@@ -377,7 +378,7 @@ enum ExtraData_e
 
 	EXTRA_SET_MVAPOOL,
 	EXTRA_SET_STRINGPOOL,
-	EXTRA_SET_MAXMATCHES,
+	EXTRA_SET_POOL_CAPACITY,
 	EXTRA_SET_MATCHPUSHED,
 	EXTRA_SET_MATCHPOPPED,
 
@@ -435,6 +436,7 @@ public:
 	CSphVector<CalcItem_t>		m_dCalcFilter;			///< items to compute for filtering
 	CSphVector<CalcItem_t>		m_dCalcSort;			///< items to compute for sorting/grouping
 	CSphVector<CalcItem_t>		m_dCalcFinal;			///< items to compute when finalizing result set
+	CSphVector<CalcItem_t>		m_dCalcPostAggregate;	///< items to compute aggregate depended with finalized result set
 
 	const CSphVector<CSphAttrOverride> *	m_pOverrides;		///< overridden attribute values
 	CSphVector<CSphAttrLocator>				m_dOverrideIn;
@@ -444,19 +446,21 @@ public:
 	CSphQueryProfile *						m_pProfile;
 	const SmallStringHash_T<int64_t> *		m_pLocalDocs;
 	int64_t									m_iTotalDocs;
+	int64_t									m_iBadRows;
 
 public:
 	CSphQueryContext ();
 	~CSphQueryContext ();
 
-	void						BindWeights ( const CSphQuery * pQuery, const CSphSchema & tSchema );
-	bool						SetupCalc ( CSphQueryResult * pResult, const ISphSchema & tInSchema, const CSphSchema & tSchema, const DWORD * pMvaPool, bool bArenaProhibit );
+	void						BindWeights ( const CSphQuery * pQuery, const CSphSchema & tSchema, CSphString & sWarning );
+	bool						SetupCalc ( CSphQueryResult * pResult, const ISphSchema & tInSchema, const CSphSchema & tSchema, const DWORD * pMvaPool, bool bArenaProhibit, bool bExtractPostAggr );
 	bool						CreateFilters ( bool bFullscan, const CSphVector<CSphFilterSettings> * pdFilters, const ISphSchema & tSchema, const DWORD * pMvaPool, const BYTE * pStrings, CSphString & sError, ESphCollation eCollation, bool bArenaProhibit, const KillListVector & dKillList );
 	bool						SetupOverrides ( const CSphQuery * pQuery, CSphQueryResult * pResult, const CSphSchema & tIndexSchema, const ISphSchema & tOutgoingSchema );
 
 	void						CalcFilter ( CSphMatch & tMatch ) const;
 	void						CalcSort ( CSphMatch & tMatch ) const;
 	void						CalcFinal ( CSphMatch & tMatch ) const;
+	void						CalcPostAggregate ( CSphMatch & tMatch ) const;
 
 	void						FreeStrFilter ( CSphMatch & tMatch ) const;
 	void						FreeStrSort ( CSphMatch & tMatch ) const;
@@ -942,6 +946,50 @@ struct PoolPtrs_t
 	{}
 };
 
+class CSphFreeList
+{
+private:
+	CSphTightVector<int>	m_dFree;
+	int						m_iNextFree;
+#ifndef NDEBUG
+	int						m_iSize;
+#endif
+
+public:
+	CSphFreeList ()
+		: m_iNextFree ( 0 )
+#ifndef NDEBUG
+		, m_iSize ( 0 )
+#endif
+	{}
+
+	void Reset ( int iSize )
+	{
+#ifndef NDEBUG
+		m_iSize = iSize;
+#endif
+		m_iNextFree = 0;
+		m_dFree.Reserve ( iSize );
+	}
+
+	int Get ()
+	{
+		int iRes = -1;
+		if ( m_dFree.GetLength () )
+			iRes = m_dFree.Pop ();
+		else
+			iRes = m_iNextFree++;
+		assert ( iRes>=0 && iRes<m_iSize );
+		return iRes;
+	}
+
+	void Free ( int iIndex )
+	{
+		assert ( iIndex>=0 && iIndex<m_iSize );
+		m_dFree.Add ( iIndex );
+	}
+};
+
 //////////////////////////////////////////////////////////////////////////
 // INLINES, FIND_XXX() GENERIC FUNCTIONS
 //////////////////////////////////////////////////////////////////////////
@@ -1250,7 +1298,7 @@ class ISphZoneCheck
 {
 public:
 	virtual ~ISphZoneCheck () {}
-	virtual SphZoneHit_e IsInZone ( int iZone, const ExtHit_t * pHit, int * pLastSpan=0 ) = 0;
+	virtual SphZoneHit_e IsInZone ( int iZone, const ExtHit_t * pHit, int * pLastSpan ) = 0;
 };
 
 
@@ -1263,7 +1311,7 @@ struct SphFactorHashEntry_t
 	SphFactorHashEntry_t *	m_pNext;
 };
 
-typedef CSphTightVector<SphFactorHashEntry_t *> SphFactorHash_t;
+typedef CSphFixedVector<SphFactorHashEntry_t *> SphFactorHash_t;
 
 
 struct SphExtraDataRankerState_t
@@ -1556,6 +1604,7 @@ public:
 	virtual BYTE *					GetToken ()										{ return m_pTokenizer->GetToken(); }
 
 	virtual ISphTokenizer *			GetEmbeddedTokenizer () const					{ return m_pTokenizer; }
+	virtual bool					WasTokenMultiformDestination ( bool & bHead, int & iDestCount ) const { return m_pTokenizer->WasTokenMultiformDestination ( bHead, iDestCount ); }
 };
 
 
@@ -1657,7 +1706,6 @@ CSphString		sphReconstructNode ( const XQNode_t * pNode, const CSphSchema * pSch
 
 void			sphSetUnlinkOld ( bool bUnlink );
 void			sphUnlinkIndex ( const char * sName, bool bForce );
-void			sphSetDebugCheck ();
 
 void			WriteSchema ( CSphWriter & fdInfo, const CSphSchema & tSchema );
 void			ReadSchema ( CSphReader & rdInfo, CSphSchema & m_tSchema, DWORD uVersion, bool bDynamic );
@@ -2278,7 +2326,7 @@ public:
 		, m_iDocStart ( 0 )
 		, m_iDocCount ( 0 )
 		, m_pExtraTokenizer ( NULL )
-		, m_bProxyStripHTML ( false )
+		, m_pProxyStripper ( NULL )
 	{
 		assert ( sphUTF8Encode ( m_pMarkerDocStart, PROXY_DOCUMENT_START )==PROXY_MARKER_LEN );
 
@@ -2297,6 +2345,7 @@ public:
 	virtual ~CSphSource_Proxy()
 	{
 		SafeDelete ( m_pExtraTokenizer );
+		SafeDelete ( m_pProxyStripper );
 	}
 
 	void AppendToField ( StoredDoc_t * pCurDoc, int iField, BYTE * pToken, int iTokenLen, BYTE * pMarker )
@@ -2338,10 +2387,10 @@ public:
 		assert ( pEmbeddedTokenizer );
 
 		// do not run the stripper twice
-		if ( CSphSource_Proxy<T>::m_bStripHTML )
+		if ( CSphSource_Proxy<T>::m_pStripper )
 		{
-			m_bProxyStripHTML = true;
-			CSphSource_Proxy<T>::m_bStripHTML = false;
+			m_pProxyStripper = CSphSource_Proxy<T>::m_pStripper;
+			CSphSource_Proxy<T>::m_pStripper = NULL;
 		}
 
 		if ( !m_pExtraTokenizer )
@@ -2412,9 +2461,9 @@ public:
 			{
 				pDoc->m_dChinese[i] = sphDetectChinese ( pFields[i], m_dFieldLengths[i] );
 
-				if ( m_bProxyStripHTML )
+				if ( m_pProxyStripper )
 				{
-					CSphSource_Proxy<T>::m_pStripper->Strip ( pFields[i] );
+					m_pProxyStripper->Strip ( pFields[i] );
 					m_dFieldLengths[i] = strlen ( (const char *)pFields[i] );
 				}
 
@@ -2570,7 +2619,7 @@ private:
 	int						m_iDocStart;
 	int						m_iDocCount;
 	ISphTokenizer *			m_pExtraTokenizer;
-	bool					m_bProxyStripHTML;
+	CSphHTMLStripper *		m_pProxyStripper;
 
 	BYTE					m_pMarkerDocStart[PROXY_MARKER_LEN];
 	BYTE					m_pMarkerChineseField[PROXY_MARKER_LEN];
@@ -2638,5 +2687,5 @@ private:
 #endif // _sphinxint_
 
 //
-// $Id: sphinxint.h 4793 2014-08-29 08:42:54Z joric $
+// $Id$
 //
