@@ -1,10 +1,10 @@
 //
-// $Id: sphinxmetaphone.cpp 4522 2014-01-30 11:00:18Z tomat $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -589,7 +589,8 @@ static int ProcessCode ( int iCode, int iCur, CurrentWord_t & Word, BYTE * sPrim
 
 void stem_dmetaphone ( BYTE * pWord )
 {
-	BYTE	sOriginal [3*SPH_MAX_WORD_LEN+3];
+	const int EXTRA_RESERVE = 16;
+	BYTE	sOriginal [3*SPH_MAX_WORD_LEN+3+EXTRA_RESERVE];
 	BYTE	sPrimary [3*SPH_MAX_WORD_LEN+3];
 	BYTE	sSecondary [ 3*SPH_MAX_WORD_LEN+3 ];
 	int		iLength = strlen ( (const char *)pWord );
@@ -661,5 +662,5 @@ void stem_dmetaphone ( BYTE * pWord )
 }
 
 //
-// $Id: sphinxmetaphone.cpp 4522 2014-01-30 11:00:18Z tomat $
+// $Id$
 //
