@@ -1,10 +1,10 @@
 //
-// $Id: sphinxrt.h 4718 2014-06-02 10:44:55Z joric $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -99,7 +99,8 @@ typedef void ProgressCallbackSimple_t ();
 
 enum ESphBinlogReplayFlags
 {
-	SPH_REPLAY_ACCEPT_DESC_TIMESTAMP = 1
+	SPH_REPLAY_ACCEPT_DESC_TIMESTAMP = 1,
+	SPH_REPLAY_IGNORE_OPEN_ERROR = 2
 };
 
 /// replay stored binlog
@@ -108,5 +109,5 @@ void sphReplayBinlog ( const SmallStringHash_T<CSphIndex*> & hIndexes, DWORD uRe
 #endif // _sphinxrt_
 
 //
-// $Id: sphinxrt.h 4718 2014-06-02 10:44:55Z joric $
+// $Id$
 //
