@@ -1,10 +1,10 @@
 //
-// $Id: sphinxstd.h 4796 2014-09-01 19:45:49Z kevg $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -277,9 +277,12 @@ inline int sphBitCount ( DWORD n )
 
 typedef			bool ( *SphDieCallback_t ) ( const char * );
 
-/// crash with an error message
+/// crash with an error message, and do not have searchd watchdog attempt to resurrect
 void			sphDie ( const char * sMessage, ... ) __attribute__ ( ( format ( printf, 1, 2 ) ) );
 
+/// crash with an error message, but have searchd watchdog attempt to resurrect
+void			sphDieRestart ( const char * sMessage, ... ) __attribute__ ( ( format ( printf, 1, 2 ) ) );
+
 /// setup a callback function to call from sphDie() before exit
 /// if callback returns false, sphDie() will not log to stdout
 void			sphSetDieCallback ( SphDieCallback_t pfDieCallback );
@@ -756,7 +759,10 @@ public:
 		if ( !iLimit )
 			iLimit = MAGIC_INITIAL_LIMIT;
 		while ( iLimit<iNewLimit )
+		{
 			iLimit *= 2;
+			assert ( iLimit>0 );
+		}
 		return iLimit;
 	}
 };
@@ -1159,9 +1165,15 @@ public:
 		if ( !iLimit )
 			iLimit = CSphVectorPolicy<T>::MAGIC_INITIAL_LIMIT;
 		while ( iLimit<iNewLimit && iLimit<SLOW_GROW_TRESHOLD )
+		{
 			iLimit *= 2;
+			assert ( iLimit>0 );
+		}
 		while ( iLimit<iNewLimit )
+		{
 			iLimit = (int)( iLimit*1.2f );
+			assert ( iLimit>0 );
+		}
 		return iLimit;
 	}
 };
@@ -1622,6 +1634,8 @@ struct CSphString
 {
 protected:
 	char *				m_sValue;
+	// Empty ("") string optimization.
+	static char EMPTY[];
 
 private:
 	/// safety gap after the string end; for instance, UTF-8 Russian stemmer
@@ -1650,7 +1664,8 @@ public:
 
 	~CSphString ()
 	{
-		SafeDeleteArray ( m_sValue );
+		if ( m_sValue!=EMPTY )
+			SafeDeleteArray ( m_sValue );
 	}
 
 	const char * cstr () const
@@ -1660,7 +1675,7 @@ public:
 
 	const char * scstr() const
 	{
-		return m_sValue ? m_sValue : "";
+		return m_sValue ? m_sValue : EMPTY;
 	}
 
 	inline bool operator == ( const char * t ) const
@@ -1689,11 +1704,17 @@ public:
 	{
 		if ( sString )
 		{
-			int iLen = 1+strlen(sString);
-			m_sValue = new char [ iLen+SAFETY_GAP ];
+			if ( sString[0]=='\0' )
+			{
+				m_sValue = EMPTY;
+			} else
+			{
+				int iLen = 1+strlen(sString);
+				m_sValue = new char [ iLen+SAFETY_GAP ];
 
-			strcpy ( m_sValue, sString ); // NOLINT
-			memset ( m_sValue+iLen, 0, SAFETY_GAP );
+				strcpy ( m_sValue, sString ); // NOLINT
+				memset ( m_sValue+iLen, 0, SAFETY_GAP );
+			}
 		} else
 		{
 			m_sValue = NULL;
@@ -1710,14 +1731,21 @@ public:
 	{
 		if ( m_sValue==rhs.m_sValue )
 			return *this;
-		SafeDeleteArray ( m_sValue );
+		if ( m_sValue!=EMPTY )
+			SafeDeleteArray ( m_sValue );
 		if ( rhs.m_sValue )
 		{
-			int iLen = 1+strlen(rhs.m_sValue);
-			m_sValue = new char [ iLen+SAFETY_GAP ];
+			if ( rhs.m_sValue[0]=='\0' )
+			{
+				m_sValue = EMPTY;
+			} else
+			{
+				int iLen = 1+strlen(rhs.m_sValue);
+				m_sValue = new char [ iLen+SAFETY_GAP ];
 
-			strcpy ( m_sValue, rhs.m_sValue ); // NOLINT
-			memset ( m_sValue+iLen, 0, SAFETY_GAP );
+				strcpy ( m_sValue, rhs.m_sValue ); // NOLINT
+				memset ( m_sValue+iLen, 0, SAFETY_GAP );
+			}
 		}
 		return *this;
 	}
@@ -1740,18 +1768,26 @@ public:
 
 	void SetBinary ( const char * sValue, int iLen )
 	{
-		SafeDeleteArray ( m_sValue );
+		if ( m_sValue!=EMPTY )
+			SafeDeleteArray ( m_sValue );
 		if ( sValue )
 		{
-			m_sValue = new char [ 1+SAFETY_GAP+iLen ];
-			memcpy ( m_sValue, sValue, iLen );
-			memset ( m_sValue+iLen, 0, 1+SAFETY_GAP );
+			if ( sValue[0]=='\0' )
+			{
+				m_sValue = EMPTY;
+			} else
+			{
+				m_sValue = new char [ 1+SAFETY_GAP+iLen ];
+				memcpy ( m_sValue, sValue, iLen );
+				memset ( m_sValue+iLen, 0, 1+SAFETY_GAP );
+			}
 		}
 	}
 
 	void Reserve ( int iLen )
 	{
-		SafeDeleteArray ( m_sValue );
+		if ( m_sValue!=EMPTY )
+			SafeDeleteArray ( m_sValue );
 		m_sValue = new char [ 1+SAFETY_GAP+iLen ];
 		memset ( m_sValue, 0, 1+SAFETY_GAP+iLen );
 	}
@@ -1845,6 +1881,13 @@ public:
 
 	char * Leak ()
 	{
+		if ( m_sValue==EMPTY )
+		{
+			m_sValue = NULL;
+			char * pBuf = new char[1];
+			pBuf[0] = '\0';
+			return pBuf;
+		}
 		char * pBuf = m_sValue;
 		m_sValue = NULL;
 		return pBuf;
@@ -1853,7 +1896,8 @@ public:
 	// opposite to Leak()
 	void Adopt ( char ** sValue )
 	{
-		SafeDeleteArray ( m_sValue );
+		if ( m_sValue!=EMPTY )
+			SafeDeleteArray ( m_sValue );
 		m_sValue = *sValue;
 		*sValue = NULL;
 	}
@@ -2087,6 +2131,7 @@ public:
 	/// copy ctor
 	CSphVariant ( const CSphVariant & rhs )
 	{
+		m_pNext = NULL;
 		*this = rhs;
 	}
 
@@ -2472,7 +2517,7 @@ public:
 		int64_t iCount = 0;
 
 #if USE_WINDOWS
-		HANDLE iFD = CreateFile ( sFile, GENERIC_READ, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0 );
+		HANDLE iFD = CreateFile ( sFile, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_READONLY, 0 );
 		if ( iFD==INVALID_HANDLE_VALUE )
 		{
 			sError.SetSprintf ( "failed to open file '%s' (errno %d)", sFile, ::GetLastError() );
@@ -3098,5 +3143,5 @@ protected:
 #endif // _sphinxstd_
 
 //
-// $Id: sphinxstd.h 4796 2014-09-01 19:45:49Z kevg $
+// $Id$
 //
