@@ -1,10 +1,10 @@
 //
-// $Id: sphinxstem.h 4522 2014-01-30 11:00:18Z tomat $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -91,8 +91,11 @@ const CSphNamedInt &	sphAotDictinfo ( int iLang );
 class CSphTokenFilter;
 CSphTokenFilter *		sphAotCreateFilter ( ISphTokenizer * pTokenizer, CSphDict * pDict, bool bIndexExact, DWORD uLangMask );
 
+/// free lemmatizers on shutdown
+void	sphAotShutdown ();
+
 #endif // _sphinxstem_
 
 //
-// $Id: sphinxstem.h 4522 2014-01-30 11:00:18Z tomat $
+// $Id$
 //
