@@ -1,10 +1,10 @@
 //
-// $Id: sphinxutils.cpp 4802 2014-09-04 06:49:49Z joric $
+// $Id$
 //
 
 //
-// Copyright (c) 2001-2014, Andrew Aksyonoff
-// Copyright (c) 2008-2014, Sphinx Technologies Inc
+// Copyright (c) 2001-2015, Andrew Aksyonoff
+// Copyright (c) 2008-2015, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -832,10 +832,6 @@ bool TryToExec ( char * pBuffer, const char * szFilename, CSphVector<char> & dRe
 	return true;
 }
 
-bool CSphConfigParser::TryToExec ( char * pBuffer, const char * szFilename, CSphVector<char> & dResult )
-{
-	return ::TryToExec ( pBuffer, szFilename, dResult, m_sError, sizeof(m_sError) );
-}
 #endif
 
 
@@ -948,7 +944,7 @@ bool CSphConfigParser::Parse ( const char * sFileName, const char * pBuffer )
 				if ( !pBuffer && m_iLine==1 && p==sBuf && p[1]=='!' )
 				{
 					CSphVector<char> dResult;
-					if ( TryToExec ( p+2, sFileName, dResult ) )
+					if ( TryToExec ( p+2, sFileName, dResult, m_sError, sizeof(m_sError) ) )
 						Parse ( sFileName, &dResult[0] );
 					break;
 				} else
@@ -1367,7 +1363,7 @@ bool sphConfIndex ( const CSphConfigSection & hIndex, CSphIndexSettings & tSetti
 			fprintf ( stdout, "WARNING: unknown docinfo=%s, defaulting to extern\n", hIndex["docinfo"].cstr() );
 
 		if ( tSettings.m_eDocinfo==SPH_DOCINFO_INLINE )
-			fprintf ( stdout, "WARNING: docinfo=inline is deprecated, use ondisc_attrs=1 instead\n" );
+			fprintf ( stdout, "WARNING: docinfo=inline is deprecated, use ondisk_attrs=1 instead\n" );
 
 		if ( tSettings.m_eDocinfo==SPH_DOCINFO_INLINE && tSettings.m_bIndexFieldLens )
 		{
@@ -2308,7 +2304,7 @@ void sphConfigureCommon ( const CSphConfig & hConf )
 		sphPluginInit ( hCommon["plugin_dir"].cstr() );
 }
 
-static bool IsChineseCode ( int iCode )
+bool sphIsChineseCode ( int iCode )
 {
 	return ( ( iCode>=0x2E80 && iCode<=0x2EF3 ) ||	// CJK radicals
 		( iCode>=0x2F00 && iCode<=0x2FD5 ) ||	// Kangxi radicals
@@ -2329,7 +2325,7 @@ bool sphDetectChinese ( const BYTE * szBuffer, int iLength )
 	while ( pBuffer<szBuffer+iLength )
 	{
 		int iCode = sphUTF8Decode ( pBuffer );
-		if ( IsChineseCode ( iCode ) )
+		if ( sphIsChineseCode ( iCode ) )
 			return true;
 	}
 
@@ -2444,5 +2440,5 @@ bool CSphDynamicLibrary::LoadSymbols ( const char **, void ***, int ) { return f
 #endif
 
 //
-// $Id: sphinxutils.cpp 4802 2014-09-04 06:49:49Z joric $
+// $Id$
 //
